/**
 * Taller 3: Fast PWM
 *
 * @brief
 *    Configuration of Timer1 in the fast PWM mode
 * @author
 *    Angel Grover Pérez Muñoz, Manuel Nieto
 */

#include "./bitwise.h" // IMPORT my bitwise operations

/***************************************************************************//**
 * L E D   F U N C T I O N S
 ******************************************************************************/

#define LED_DDRX   (DDRC)
#define LED_DDRXn  (DDC7)
#define LED_PORTX  (PORTC)
#define LED_PORTXn (PORTC7)

static void led_setup() {
  sbi(LED_DDRX, LED_DDRXn);
}

static void led_toggle() {
  static bool on = false;
  if (on) {
    cbi(LED_PORTX, LED_PORTXn); //< turn OFF (Put output to 0, 0V)
    Serial.println("Led OFF");       
    on = false;
  } else {
    sbi(LED_PORTX, LED_PORTXn); //< turn ON  (Put output to 1, 5V)
    Serial.println("Led ON");
    on = true;
  }
}

/***************************************************************************//**
 * T I M E R 1   F U N C T I O N S
 ******************************************************************************/

/**
 * Timer1 pins:
 */
// Channel B - D10
#define D10_DDRX  (DDRB)
#define D10_DDRXn (DDB6)
// Channel C - D11
#define D11_DDRX  (DDRB)
#define D11_DDRXn (DDB7)

/**
 * Software counters for the Timer1.
 */
volatile int64_t t1_count_a = 0; //< Used as TOP
volatile int64_t t1_count_b = 0;
volatile int64_t t1_count_c = 0;

/**
 * @brief Configure timer 1 in Fast PWM mode with OCR1A as the TOP
 * Example for 20ms or 50 Hz
 */
static void timer_setup() {
  cli();
  TCCR1A = 0x00;
  TCCR1B = 0x00;
  TCCR1C = 0x00;
  TIMSK1 = 0x00;

  // 1. Pin Configuration
  // Configure the D10 (OC1B) and D11 (OC1C) as output pin via DDR.
  /** @todo */
  // Channel B (D10) and C (D11): Clear on compare match & set at top.
  /** @todo */

  // 2. Setup mode: Fast PWM & OCR1A as TOP (channel A can't be used for PWM)
  /** @todo */

  // 3. Setup TOP value
  /** @todo */

  // 4. Setup initial comparision register values
  /** @todo */

  // 5. Prescaler
  /** @todo */

  // 6. Setup IRQs. Enable IRQ when TCNT1 matches OCR1A | OCR1B | OCR1C
  /** @todo */

  sei();
}

/**
 * @brief
 *    This ISR is invoked at each match for channel A, when TCNT1 == OCR1A.
 */
ISR(TIMER1_COMPA_vect) {
  t1_count_a++;
}

/**
 * @brief
 *    This ISR is invoked at each match for channel B, when TCNT1 == OCR1B.
 */
ISR(TIMER1_COMPB_vect) {
  t1_count_b++;
}

/**
 * @brief
 *    This ISR is invoked at each match for channel C, when TCNT1 == OCR1C.
 */
ISR(TIMER1_COMPC_vect) {
  t1_count_c++;
}

#define CH_B (0)
#define CH_C (1)
static void set_pwm_duty(int ch, float dc) {
  /** @todo */
}

/***************************************************************************//**
 * A R D U I N O   F U N C T I O N S
 ******************************************************************************/

/** @brief Runs once when you press reset or power the board
 */
void setup() {
  Serial.begin(57600); //< Configure the serial port
  led_setup();         //< Configure the built-in LED
  while(!Serial);      //< So that Serial Monitor window has time to initializeinitialize
  timer_setup();       //< Configure the Timer1
}

/** @brief Runs over and over again forever
 */
void loop() {
  Serial.println("From 0% to 100%");
  delay(5000);
  float dc = 0.0; // duty cycle in %
  for (dc = 0.0; dc <= 100.0; dc += 1.0) {
    set_pwm_duty(CH_B, dc);
    set_pwm_duty(CH_C, dc);
    delay(100);
  }
  led_toggle();

  Serial.println("From 100.0% to 0.0%");
  for (dc = 100.0; dc >= 0.0; dc -= 1.0) {
    set_pwm_duty(CH_B, dc);
    set_pwm_duty(CH_C, dc);
    delay(100);
  }
  led_toggle();
}